Module algorithm

Module algorithm 

Graph algorithms.

Functions

lowest_common_ancestor
Returns the lowest common ancestor of the given nodes in the graph.
shortest_path_length
Returns the length of the shortest path between two nodes in the graph.