Builder

Struct Builder 

pub struct Builder<T> { /* private fields */ }
Graph builder.

Implementations

Source

impl<T> Builder<T>

pub fn add_node(&mut self, data: T) -> usize

Adds a node to the graph.

Examples
use zrx_graph::Graph;

// Create graph builder and add nodes
let mut builder = Graph::builder();
let a = builder.add_node("a");
let b = builder.add_node("b");
pub fn add_edge(&mut self, source: usize, target: usize) -> Result

Adds an edge to the graph.

Errors

In case the source or target node doesn’t exist, Error::NotFound is returned, to make sure the graph does not contain stale node references. By returning an error instead of panicking, we can provide recoverable and proper error handling to the caller.

This is mentionable, as some other graph libraries will just panic and crash the program, like the popular petgraph crate. Additionally, note that this method does not check whether an edge already exists, as the existence of multiple edges is a valid use case in some scenarios.

Examples
use zrx_graph::Graph;

// Create graph builder and add nodes
let mut builder = Graph::builder();
let a = builder.add_node("a");
let b = builder.add_node("b");
let c = builder.add_node("c");

// Create edges between nodes
builder.add_edge(a, b)?;
builder.add_edge(b, c)?;
pub fn iter(&self) -> Range<usize>

Creates an iterator over the graph builder.

This iterator emits the node indices, which is exactly the same as iterating over the adjacency list using 0..self.len().

Examples
use zrx_graph::topology::Adjacency;
use zrx_graph::Graph;

// Create graph builder and add nodes
let mut builder = Graph::builder();
let a = builder.add_node("a");
let b = builder.add_node("b");
let c = builder.add_node("c");

// Create edges between nodes
builder.add_edge(a, b)?;
builder.add_edge(b, c)?;

// Create iterator over graph builder
for node in builder.iter() {
    println!("{node:?}");
}
pub fn build(self) -> Graph<T>

Builds the graph.

This method creates the actual graph from the builder, which brings the graph into an executable form to allow for very efficient traversal.

Examples
use zrx_graph::Graph;

// Create graph builder and add nodes
let mut builder = Graph::builder();
let a = builder.add_node("a");
let b = builder.add_node("b");
let c = builder.add_node("c");

// Create edges between nodes
builder.add_edge(a, b)?;
builder.add_edge(b, c)?;

// Create graph from builder
let graph = builder.build();
Source

impl<T> Builder<T>

pub fn nodes(&self) -> &[T]

Returns a reference to the nodes.

pub fn edges(&self) -> &[Edge]

Returns a reference to the edges.

pub fn len(&self) -> usize

Returns the number of nodes.

pub fn is_empty(&self) -> bool

Returns whether there are any nodes.