pub struct Distance { /* private fields */ }Expand description
Distance matrix.
This data type stores the all-pairs shortest path distances for all nodes in a directed acyclic graph (DAG). It’s computed through the Floyd-Warshall algorithm, and allows for efficient retrieval of distances between any two nodes, which is essential for many graph algorithms.
Implementations
Sourceimpl Distance
impl Distance
Sourcepub fn new(adj: &Adjacency) -> Self
pub fn new(adj: &Adjacency) -> Self
Creates a distance matrix.
This method is called by Topology::new, and is not intended to be
used on its own, since an adjacency list is needed to create the matrix.
Computation is expensive, which is why Topology will defer creation
via OnceCell, so it’s only computed when first accessed.