Distance

Struct Distance 

pub struct Distance { /* private fields */ }
Distance matrix.

This data type stores the all-pairs shortest path distances for all nodes in a directed acyclic graph (DAG). It’s computed through the Floyd-Warshall algorithm, and allows for efficient retrieval of distances between any two nodes, which is essential for many graph algorithms.

Implementations

Source

impl Distance

pub fn new<T, W>(builder: &Builder<T, W>) -> Self
where W: Clone,

Creates a distance matrix.

Examples
use zrx_graph::topology::Distance;
use zrx_graph::Graph;

// Create graph builder and add nodes
let mut builder = Graph::builder();
let a = builder.add_node("a");
let b = builder.add_node("b");
let c = builder.add_node("c");

// Create edges between nodes
builder.add_edge(a, b, 0)?;
builder.add_edge(b, c, 0)?;

// Create distance matrix
let dist = Distance::new(&builder);
assert_eq!(dist[a][c], 2);